@@ -1,3 +1,4 @@
+# This is an exmaple of sqlpygen file
 -- module: example1
 
 -- schema: table_stocks
@@ -16,6 +17,11 @@ CREATE TABLE stocks (
 INSERT INTO stocks VALUES (:date, :trans, :symbol, :qty, :price) ;
 
 -- query: select_from_stocks
--- return: str, str, str, float, float
+-- return*: str, str, str, float, float
 
 SELECT * FROM stocks ;
+
+-- query: count_stocks
+-- return?: int!
+
+SELECT COUNT(*) FROM stocks ;
